import re

import numpy as np


def load():
    file_info = open ("info_model.txt", 'r')
    if file_info == None:
        print ("Info File \'info_model.txt\' is absent")
        return False

    # parse info file
    data = file_info.read ().split ("\n")
    for i, dat in enumerate (data):
        # words = [words.split(" ") for words in dat]
        if i == 0:
            rows = int (''.join (re.findall (r'\d', dat)))
            # print(rows)
        if i == 1:
            columns = int (''.join (re.findall (r'\d', dat)))
            # print(columns)
        if i == 3:
            file_name_matrix = re.match (r'^\w+\.csv', dat).group (0)
            # print(file_name_matrix)
        if i == 5:
            file_name_dictionary = re.match (r'^\w+\.txt', dat).group (0)
            # print(file_name_dictionary)
        if i == 7:
            file_name_tags = re.match (r'^\w+\.txt', dat).group (0)
            # print(file_name_tags)

    # read matrix
    matrix = np.genfromtxt (file_name_matrix, delimiter=';')
    # print(matrix)

    # read tags !
    tags = [item[:-1] for item in open (file_name_tags)]
    # print(tags)

    # read dictionary
    dictionary = [word[:-1] for word in open (file_name_dictionary)]
    # print(dictionary)
    return rows, columns, dictionary, tags, matrix


def save():
    """ Just not used function. template from news_classify_tags.py """
    import os

    # save info data
    file_info = open ("info_model.txt", 'w')
    if file_info == None:
        return False
    file_info.write ("number of features = " + str (len (self.vectorizer.get_feature_names ())) + '\n')
    file_info.write ("Quantity of tags = " + str (len (self.tags)) + '\n')
    file_info.write ("\ndata_tag_frequency.csv  ---- Model data matrix --- " + \
                     str (self.trained_model.shape) + "\n")
    file_info.write ("\nDictionary.txt -- Total number words in  is---" + str (
        len (self.vectorizer.get_feature_names ())) + '\n')
    file_info.write ("\ntags.txt --- Total number is ---" + str (len (self.tags)) + '\n')
    file_info.close ()

    # save model matrix
    file_tag_frequency = 'data_tag_frequency.csv'
    if os.path.isfile (file_tag_frequency):
        os.remove (file_tag_frequency)
    file_data = open (file_tag_frequency, "w")
    if file_data == None:
        print ("Can't create data storage file")
        return False
    np.savetxt (file_tag_frequency, self.tag_frequency_matrix, delimiter=';', fmt='%1.4f')
    print ("Data is saved into file: " + file_tag_frequency + " " + str (
        int (os.stat (file_tag_frequency).st_size / 1024)) + 'kB')

    # save dictionary
    file_info_dictionaries = open ("Dictionary.txt", "w")
    if file_info_dictionaries == None:
        return False
    for line in self.vectorizer.get_feature_names ():
        file_info_dictionaries.write (line + "\n")
    file_info_dictionaries.close ()

    # save tags
    file_info_tags = open ("tags.txt", "w")
    if file_info_tags == None:
        return False
    for text1, text2 in self.tags:
        file_info_tags.write (text1 + " " + text2.decode ('ascii', "ignore") + "\n")
    file_info_tags.close ()
    return True


def graph_results(in_data, title=''):
    # import numpy as np
    from pylab import figure, hist, savefig, show
    data = np.asarray(in_data)
    figure()

    hist(data, bins=50)
    show()
    savefig("fig.png")


def text_clear(word):
    from stemming.porter2 import stem

    texts = word.lower()
    texts = stem(texts)
    return texts


if __name__ == '__main__':
    rows, columns, data_dictionary, tags, matrix = load ()
    if data_dictionary is None:
        print ("Something wrong whit data files. ")
        exit (1)
    # print(rows, columns, data_dictionary, tags, matrix )
    np_matrix = np.asarray(matrix)
    # print(np_matrix)
    print("Empty taggers are :")
    tags_empty = [tag for tag in range(columns-1, 0, -1) if sum (np_matrix[tag]) == 0]
    print(len(tags_empty), "are empty of ", len(tags))

    # erase "empty" tags and rows in matrix
    for i in tags_empty:
        del tags[i]
        print(i, end=",")
    np_matrix = np.delete(np_matrix, tags_empty, axis=0)
    print('\n')
    print('Now tags are', len(tags))
    print(tags)
    print(np_matrix.shape)
    np_transpose = np_matrix.transpose()
    # print(np_transpose.shape)
    # print(np_transpose[2][25])
    for i in range(np_transpose.shape[0]):
        for j in range(np_transpose.shape[1]):
            if np_transpose[i][j] != 0:
                np_transpose[i][j] = 1
                # print(i,j)

    # words frequency measurement
    words_frequency={}
    for row in range(np_transpose.shape[0]):
        # print(row, sum(np_transpose[row]), data_dictionary[row])
        words_frequency[data_dictionary[row]]= (int(sum(np_transpose[row])))
    # print(words_frequency)

    # !!!!graph results
    frequency = list(words_frequency.values())
    graph_results(frequency, "The Words Frequencies")

    # the are some words with great frequency

    very_frequent_words = dict([(key, value) for key, value in words_frequency.items() if value > 100])
    print('There are ', len(very_frequent_words), ' very frequent words: ')
    for item in sorted(very_frequent_words, key=very_frequent_words.get, reverse=True):
        print(item, ":", very_frequent_words[item], end=", ")

    un_frequent_words = dict([(key, value) for key, value in words_frequency.items() if value < 10])
    print('\nThere are ', len(un_frequent_words), ' very seldom words: ')
    for item in sorted(un_frequent_words, key=un_frequent_words.get, reverse=True):
        print(item, ":", un_frequent_words[item], end=", ")

    # compare words set from news to words of DICTIONARY
    dict_standart = set([text_clear(word) for word in open('large.txt', 'r').read().split('\n')])
    dict_standart1 = set([word for word in open('large.txt', 'r').read().split('\n')])
    # print(len(dict_standart), dict_standart)
    diff_dict_sets = set(data_dictionary) - dict_standart
    print('\nThe difference of extracted dictionary and common dictionary is', len(diff_dict_sets))
    print(diff_dict_sets)
